/*
   This include file is to be used with the Optrex DMC-16202-LY LCD. It is set
   to accept data entry in 4-bit values on the LCD lines D7, D6, D5, D4 with D4
   being the LSB and D7 being the MSB. The LCD_RS line is used to distinguish
   between control of address registers and data registers. The address
   registers are used to setup the LCD for different function modes such as
   cursor on/off, display on/off, line 1 or 2 etc... The data register is used
   to hold characters for dislay. The LCD_RS line is set HIGH or LOW for desired
   control. The LCD_EN line is used to pulse each registers data into the LCD.
   There are certain timing requirements that must be met. Such requirements are
   minimum 15ms after power-up before initialization, 5ms after initialization,
   and 125us after each LCD_EN pulse. Changing any of these timings may result
   in improper operation. LCD_String_1 is used to hold the character array for
   the LCD line 1, and LCD_String_2 is used to hold the character array for the
   LCD line 2. These variables are global and can be filled any where after this
   include file is placed in the program. It is a good idea to use the CLear_LCD
   routine between LCD value updates to ensure all unwanted characters are
   removed from the display.


         WARNING:
            Linx Technologies is NOT responsible for any damage that
            may occur from any use of this code. It has been used and
            tested with the Linx MS series encoders/decoders and has
            been found to be completely functional as per the
            corresponding data guides and app notes that apply.






   // These pin outputs must be named this way in the .C file
   #define LCD_D4       // Bit-0 of 4-bit data entry
   #define LCD_D5       // Bit-1 of 4-bit data entry
   #define LCD_D6       // Bit-2 of 4-bit data entry
   #define LCD_D7       // Bit-3 of 4-bit data entry
   #define LCD_RS       // Register Select line of LCD
   #define LCD_EN       // Enable line of LCD
*/



#include <stdio.h>
#include <string.h>

//Constants
const byte LCD_Init_Values[4] = {0x02, 0x28, 0x0C, 0x06};
const byte Line_1 = 0x80;        // Address value for LCD Line 1
const byte Line_2 = 0xC0;        // Address value for LCD Line 2

// Variables
byte i;                          // Stores Array index values
char LCD_String_1[17];           // Character Array for line 1 of LCD
char LCD_String_2[17];   		   // Character Array for line 2 of LCD


//******************************************************************************






//******************************************************************************
void Pulse_EN_Line()
{
   OUTPUT_HIGH(LCD_EN);          // Set LCD Enable line HIGH
   delay_cycles(2);              // Wait a couple of clock cycles for pulse time
   OUTPUT_LOW(LCD_EN);           // Clear LCD Enable line LOW
   delay_us(125);                // Wait 125uS for LCD to register pulse
}
//******************************************************************************


//******************************************************************************
void LCD_Send_Byte(byte Value)
{
   OUTPUT_BIT(LCD_D7, BIT_TEST(Value,7));
   OUTPUT_BIT(LCD_D6, BIT_TEST(Value,6));
   OUTPUT_BIT(LCD_D5, BIT_TEST(Value,5));
   OUTPUT_BIT(LCD_D4, BIT_TEST(Value,4));

   Pulse_EN_Line();              // Pulse the LCD Enable line to register value

   OUTPUT_BIT(LCD_D7, BIT_TEST(Value,3));
   OUTPUT_BIT(LCD_D6, BIT_TEST(Value,2));
   OUTPUT_BIT(LCD_D5, BIT_TEST(Value,1));
   OUTPUT_BIT(LCD_D4, BIT_TEST(Value,0));

   Pulse_EN_Line();              // Pulse the LCD Enable line to register value
}
//******************************************************************************


//******************************************************************************
void LCD_Init()
{
   delay_ms(100);                // Wait 100mS after LCD power-up (15mS needed)

   OUTPUT_LOW(LCD_RS);           // Clear LCD Register Select to set-up Address
   delay_cycles(2);              // Wait a couple of clock cycles for pulse time
   OUTPUT_LOW(LCD_EN);           // Clear LCD Enable line LOW
   delay_us(125);                // Wait 125uS for LCD to register pulse

   for(i=0; i<=3; i++)           // Inc. i from 0 to 3 to send all 4 Init. bytes
      LCD_Send_Byte(LCD_Init_Values[i]);

   delay_ms(10);                 // Wait 10mS for LCD to initialize (5mS needed)
}
//******************************************************************************


//******************************************************************************
void LCD_Line_1()
{
   OUTPUT_LOW(LCD_RS);           // Clear LCD Register Select to set-up Address
   delay_cycles(2);              // Wait a couple of clock cycles for pulse time
   OUTPUT_LOW(LCD_EN);           // Clear LCD Enable line LOW
   delay_us(125);                // Wait 125uS for LCD to register pulse

   LCD_Send_Byte(Line_1);        // Send address indicating LCD Line 1 (0x80)

   OUTPUT_HIGH(LCD_RS);          // Set LCD Register Select to set-up Data
   delay_cycles(2);              // Wait a couple of clock cycles for pulse time
   OUTPUT_LOW(LCD_EN);           // Clear LCD Enable line LOW
   delay_us(125);                // Wait 125uS for LCD to register pulse

                                 // Increment i from 0 to length of LCD_String_1
   for(i=0; i<=(strlen(LCD_String_1) - 1); i++)
      LCD_Send_Byte(LCD_String_1[i]);
}
//******************************************************************************


//******************************************************************************
void LCD_Line_2()
{
   OUTPUT_LOW(LCD_RS);           // Clear LCD Register Select to set-up address
   delay_cycles(2);              // Wait a couple of clock cycles for pulse time
   OUTPUT_LOW(LCD_EN);           // Clear LCD Enable line LOW
   delay_us(125);                // Wait 125uS for LCD to register pulse

   LCD_Send_Byte(Line_2);        // Send address indicating LCD Line 2 (0xC0)

   OUTPUT_HIGH(LCD_RS);          // Set LCD Register Select to set-up Data
   delay_cycles(2);              // Wait a couple of clock cycles for pulse time
   OUTPUT_LOW(LCD_EN);           // Clear LCD Enable line LOW
   delay_us(125);                // Wait 125uS for LCD to register pulse

                                 // Increment i from 0 to length of LCD_String_1
   for(i=0; i<=(strlen(LCD_String_2) - 1); i++)
      LCD_Send_Byte(LCD_String_2[i]);
}
//******************************************************************************


//******************************************************************************
void Clear_LCD()
{
   strcpy(LCD_String_1, "                ");
   strcpy(LCD_String_2, "                ");

   LCD_Line_1();                 // Put LCD_String_1 on LCD line 1
   LCD_Line_2();                 // Put LCD_String_2 on LCD line 2
}
//******************************************************************************

