/*
********************************************************************************

	This firmware is a demo program that illustrates how to interface a
   microprocessor to a LCD to display the encoder's TX_ID from the decoder.
   It is intended to only be used with the MS series encoders and decoders.
   It uses a PIC16F88 MCU and an Optrex DMC-16202-LY LCD. The firmware is
   written in C, compiled with a CCS C compiler. Be sure to include all the 
   necessary files when building this project. 


         WARNING:
            Linx Technologies is NOT responsible for any damage that
            may occur from any use of this code. It has been used and
            tested with the Linx MS series encoders/decoders and has
            been found to be completely functional as per the
            corresponding data guides and app notes that apply.




(c) Copyright 2006
All Rights Reserved
Linx Technologies
575 SE Ashley Place
Grants Pass, Oregon 97526

CONFIDENTIAL
This firmware source code is proprietary and is the property of Linx Technology.

********************************************************************************


***********************************
**    Enc_Dec_MDEV_Board         **
**    Written By:                **
**    Chris Murphy               **
**    Linx Technologies          **
***********************************


********************************************************************************
Purpose and Scope
   This firmware is used to interface between the Decoder and the USB module for
   use with the Master Development Software for the MS encoders and decoders.
********************************************************************************


 	 	Program memory: 4096x14  Data RAM: 367  Stack: 8
		I/O: 16   Analog Pins: 1
		Data EEPROM: 256
		C Scratch area: 77   ID Location: 2000
		Fuses: LP,XT,HS,EC_IO,NOWDT,WDT,NOPUT,PUT,MCLR,NOMCLR,BROWNOUT
		Fuses: NOBROWNOUT,LVP,NOLVP,CPD,NOCPD,WRT,NOWRT,DEBUG,NODEBUG,CCPB0
		Fuses: CCPB3,PROTECT,NOPROTECT,INTRC,INTRC_IO,RC,RC_IO



                            PIC16LF88-I/SS
                    +----------------------------+
       DataLine3  ->|  1 RA2              RA1 18 |<-  DataLine4
       DataLine2  ->|  2 RA3              RA0 17 |<-  DataLine5
             GND  ->|  3 RA4              RA7 16 |<-  DataLine6
             GND  ->|  4 Mclr             RA6 15 |<-  DataLine7
             GND  ->|  5 GND              VDD 14 |<-  +5V
       DataLine1  ->|  6 RB0              RB7 13 |<-  LatchMode
       DataLine0  ->|  7 RB1              RB6 12 |<-  GND
       EncoderID  ->|  8 RX                TX 11 |->  TX_DATA
             GND  ->|  9 RB3              RB4 10 |<-  GND
                    +----------------------------+
*/


#include <16F88.h>
#include <stdio.h>
#fuses INTRC_IO,NOMCLR,NOWDT,NOPUT,NOBROWNOUT,NOLVP,CPD,PROTECT,NOWRT,NODEBUG
#use delay(clock=8000000)
#use rs232(baud=2400, xmit=PIN_B5, parity=N, bits=8)

// Inputs
#define DataLine0 PIN_B1
#define DataLine1 PIN_B0
#define DataLine2 PIN_A3
#define DataLine3 PIN_A2
#define DataLine4 PIN_A1
#define DataLine5 PIN_A0
#define DataLine6 PIN_A7
#define DataLine7 PIN_A6
#define LatchMode PIN_B7
#define EncoderID PIN_B2


// Variables
int  Cntr = 0;
int  User_ID = 0;
int  Data_Out = 0;
int  Latched = 'M';
long Bit_Cntr = 0;
long LO_1 = 0;
long HI_1 = 0;
long LO_2 = 0;
long HI_2 = 0;
long LO_3 = 0;
long HI_3 = 0;
long LO_4 = 0;
long HI_4 = 0;


//******************************************************************************
void Seperate_Bits(int Bit_Size)
{
   Cntr = (Bit_Size / 2);
   while(((LO_1 % Bit_Size) != 0) && (Cntr != 0)) {LO_1 += 1; Cntr--;}
   LO_1 = (LO_1 / Bit_Size);

   Cntr = (Bit_Size / 2);
   while(((HI_1 % Bit_Size) != 0) && (Cntr != 0)) {HI_1 += 1; Cntr--;}
   HI_1 = (HI_1 / Bit_Size);

   Cntr = (Bit_Size / 2);
   while(((LO_2 % Bit_Size) != 0) && (Cntr != 0)) {LO_2 += 1; Cntr--;}
   LO_2 = (LO_2 / Bit_Size);

   Cntr = (Bit_Size / 2);
   while(((HI_2 % Bit_Size) != 0) && (Cntr != 0)) {HI_2 += 1; Cntr--;}
   HI_2 = (HI_2 / Bit_Size);

   Cntr = (Bit_Size / 2);
   while(((LO_3 % Bit_Size) != 0) && (Cntr != 0)) {LO_3 += 1; Cntr--;}
   LO_3 = (LO_3 / Bit_Size);

   Cntr = (Bit_Size / 2);
   while(((HI_3 % Bit_Size) != 0) && (Cntr != 0)) {HI_3 += 1; Cntr--;}
   HI_3 = (HI_3 / Bit_Size);

   Cntr = (Bit_Size / 2);
   while(((LO_4 % Bit_Size) != 0) && (Cntr != 0)) {LO_4 += 1; Cntr--;}
   LO_4 = (LO_4 / Bit_Size);

   Cntr = (Bit_Size / 2);
   while(((HI_4 % Bit_Size) != 0) && (Cntr != 0)) {HI_4 += 1; Cntr--;}
   HI_4 = (HI_4 / Bit_Size);
}
//******************************************************************************


//******************************************************************************
int Test_Bits()
{
   switch (LO_1)
   {
      case(6)  :  return(32); break;
      case(5)  :  if(HI_1 == 1) return(16); break;
      case(4)  :  if(HI_1 == 2) return(24);
                  else if(HI_1 == 1) {
                     if     (LO_2 >= 2) return(8);
                     else if(LO_2 == 1) return(40); }
                  break;
      case(3)  :  if     (HI_1 == 3) return(28);
                  else if(HI_1 == 2) return(12);
                  else if(HI_1 == 1) {
                     if     (LO_2 >= 3) return(4);
                     else if(LO_2 == 2) return(36);
                     else if(LO_2 == 1) return(20); }
                  break;
      case(2)  :  if     (HI_1 == 4) return(30);
                  else if(HI_1 == 3) return(14);
                  else if(HI_1 == 2) {
                     if     (LO_2 >= 3) return(6);
                     else if(LO_2 == 2) return(38);
                     else if(LO_2 == 1) return(22); }
                  else if(HI_1 == 1) {
                     if     (LO_2 >= 4) return(2);
                     else if(LO_2 == 3) return(34);
                     else if(LO_2 == 2) return(18);
                     else if(LO_2 == 1) {
                        if     (HI_2 == 2) return(26);
                        else if(HI_2 == 1) return(10); }}
                  break;
      case(1)  :  if     (HI_1 == 5) return(31);
                  else if(HI_1 == 4) return(15);
                  else if(HI_1 == 3) {
                     if     (LO_2 >= 3) return(7);
                     else if(LO_2 == 2) return(39);
                     else if(LO_2 == 1) return(23); }
                  else if(HI_1 == 2) {
                     if     (LO_2 >= 4) return(3);
                     else if(LO_2 == 3) return(35);
                     else if(LO_2 == 2) return(19);
                     else if(LO_2 == 1) {
                        if     (HI_2 == 2) return(27);
                        else if(HI_2 == 1) return(11); }}
                  else if(HI_1 == 1) {
                     if     (LO_2 >= 5) return(1);
                     else if(LO_2 == 4) return(33);
                     else if(LO_2 == 3) return(17);
                     else if(LO_2 == 2) {
                        if     (HI_2 >= 2) return(25);
                        else if(HI_2 >= 1) return(9); }
                     else if(LO_2 == 1) {
                        if     (HI_2 == 3) return(29);
                        else if(HI_2 == 2) return(13);
                        else if(HI_2 == 1) {
                           if     (LO_3 >= 3) return(5);
                           else if(LO_3 == 2) return(37);
                           else if(LO_3 == 1) return(21); }}}
                  break;
   }
   return(0);
}
//******************************************************************************


//******************************************************************************
void main(void)
{
   // Setup chip
   SETUP_OSCILLATOR(OSC_8MHZ);
   SETUP_ADC(ADC_OFF);
   SETUP_ADC_PORTS(NO_ANALOGS);
   SETUP_COMPARATOR(NC_NC_NC_NC);
   SETUP_SPI(SPI_SS_DISABLED);
   DISABLE_INTERRUPTS(GLOBAL);

   // Clear all I/O's
   OUTPUT_A(0x00);
   OUTPUT_B(0x00);

   // Setup all I/O's
   SET_TRIS_A(0xFF);
   SET_TRIS_B(0xDF);

   // Start the program
   while(TRUE)
   {
      LO_1 = 0;
      HI_1 = 0;
      LO_2 = 0;
      HI_2 = 0;
      LO_3 = 0;
      HI_3 = 0;
      LO_4 = 0;
      HI_4 = 0;
      User_ID = 0;
      Bit_Cntr = 0;

      while(!INPUT(EncoderID)) {}
      while(INPUT(EncoderID)) {}

      while((!INPUT(EncoderID)) && (LO_1 < 400)) LO_1 += 1;
      while(( INPUT(EncoderID)) && (HI_1 < 400)) HI_1 += 1;
      if(HI_1 != 400) {
         while((!INPUT(EncoderID)) && (LO_2 < 400)) LO_2 += 1;
         while(( INPUT(EncoderID)) && (HI_2 < 400)) HI_2 += 1;
         if(HI_2 != 400) {
            while((!INPUT(EncoderID)) && (LO_3 < 400)) LO_3 += 1;
            while(( INPUT(EncoderID)) && (HI_3 < 400)) HI_3 += 1;
            if(HI_3 != 400) {
               while((!INPUT(EncoderID)) && (LO_4 < 400)) LO_4 += 1;
               while(( INPUT(EncoderID)) && (HI_4 < 400)) HI_4 += 1;
            }
         }
      }

      Bit_Cntr = ((LO_1 + HI_1 + LO_2 + HI_2 + LO_3 + HI_3 + LO_4 + HI_4) - 400);

      if     (Bit_Cntr > 300) { Seperate_Bits(45); User_ID = Test_Bits(); }
      else if(Bit_Cntr > 150) { Seperate_Bits(23); User_ID = Test_Bits(); }
      else if(Bit_Cntr > 75)  { Seperate_Bits(12); User_ID = Test_Bits(); }
      else if(Bit_Cntr > 45)  { Seperate_Bits(6);  User_ID = Test_Bits(); }
      else if(Bit_Cntr > 20)  { Seperate_Bits(4);  User_ID = Test_Bits(); }

      if(INPUT(LatchMode)) Latched='L'; else Latched='M';
      do {
         Data_Out = 0;
         if(INPUT(DataLine0)) Data_Out += 1;
         if(INPUT(DataLine1)) Data_Out += 2;
         if(INPUT(DataLine2)) Data_Out += 4;
         if(INPUT(DataLine3)) Data_Out += 8;
         if(INPUT(DataLine4)) Data_Out += 16;
         if(INPUT(DataLine5)) Data_Out += 32;
         if(INPUT(DataLine6)) Data_Out += 64;
         if(INPUT(DataLine7)) Data_Out += 128;
         if(User_ID > 0){
            putchar(Latched);
            putchar(Data_Out);
            putchar(Data_Out);
            putchar(User_ID);
         }
      }  while(INPUT(EncoderID));
   }
}
//******************************************************************************
