/*
	This firmware is a demo program that illustrates how to interface a
   microprocessor to a LCD to display the encoder's TX_ID from the decoder.
   It is intended to only be used with the MS series encoders and decoders.
   It uses a PIC16F88 MCU and an Optrex DMC-16202-LY LCD. The firmware is
   written in C, compiled with a CCS C compiler. Be sure to include all the 
   necessary files when building this project. 


         WARNING:
            Linx Technologies is NOT responsible for any damage that
            may occur from any use of this code. It has been used and
            tested with the Linx MS series encoders/decoders and has
            been found to be completely functional as per the
            corresponding data guides and app notes that apply.










                              PIC16LF88-I/SS
                    +---------------------------+
              D6  <-|  1 RA2             RA1 18 |->  D5
              D7  <-|  2 RA3             RA0 17 |->  D4
             GND  <-|  3 RA4             RA7 16 |->  EN
             GND  <-|  4 Mclr            RA6 15 |->  RS
             GND  --|  5 GND             VDD 14 |--  +5V
             GND  <-|  6 RB0             RB7 13 |->  GND
             GND  <-|  9 RB1             RB6 12 |->  GND
           TX_ID  ->|  7 RX              RB5 11 |->  GND
             GND  <-|  9 RB3             RB4 10 |->  GND
                    +---------------------------+
*/





#include <16F88.h>
#fuses INTRC_IO,NOMCLR,NOWDT,NOPUT,NOBROWNOUT,NOLVP,NOCPD,NOPROTECT
#use delay(clock=4000000)
#use rs232(baud=2400, xmit=PIN_B5, rcv=PIN_B2, parity=N, bits=8)

// Outputs
#define LCD_D4          PIN_A0      // Bit-0 of 4-bit data entry
#define LCD_D5          PIN_A1      // Bit-1 of 4-bit data entry
#define LCD_D6          PIN_A2      // Bit-2 of 4-bit data entry
#define LCD_D7          PIN_A3      // Bit-3 of 4-bit data entry
#define LCD_RS          PIN_A6      // Register Select line of LCD
#define LCD_EN          PIN_A7      // Enable line of LCD

#include <4-bit_LCD.inc>            // File contains functions for LCD control



//******************************************************************************
//    This function assigns a number to the 8-bit data pattern of the decoder's
// User_ID. It is incremented as each new User is added to the decoders memory
// bank. To change the naming convention simply replace the text inside the
// quotation marks, which is currently labled "User# ?" (in the SWITCH routine)
// with a name that best fits your application.

void Interpret_User_ID(byte ID)
{
      // Set string value for LCD Line 1
      strcpy(LCD_String_1, "User ID:");

      // Set string value for LCD Line 2
   switch (ID)
   {
      case(0) : strcpy(LCD_String_2, "No valid data");   break;
      case(1) : strcpy(LCD_String_2, "User# 1");         break;
      case(2) : strcpy(LCD_String_2, "User# 2");         break;
      case(3) : strcpy(LCD_String_2, "User# 3");         break;
      case(4) : strcpy(LCD_String_2, "User# 4");         break;
      case(5) : strcpy(LCD_String_2, "User# 5");         break;
      case(6) : strcpy(LCD_String_2, "User# 6");         break;
      case(7) : strcpy(LCD_String_2, "User# 7");         break;
      case(8) : strcpy(LCD_String_2, "User# 8");         break;
      case(9) : strcpy(LCD_String_2, "User# 9");         break;
      case(10) : strcpy(LCD_String_2, "User# 10");       break;
      case(11) : strcpy(LCD_String_2, "User# 11");       break;
      case(12) : strcpy(LCD_String_2, "User# 12");       break;
      case(13) : strcpy(LCD_String_2, "User# 13");       break;
      case(14) : strcpy(LCD_String_2, "User# 14");       break;
      case(15) : strcpy(LCD_String_2, "User# 15");       break;
      case(16) : strcpy(LCD_String_2, "User# 16");       break;
      case(17) : strcpy(LCD_String_2, "User# 17");       break;
      case(18) : strcpy(LCD_String_2, "User# 18");       break;
      case(19) : strcpy(LCD_String_2, "User# 19");       break;
      case(20) : strcpy(LCD_String_2, "User# 20");       break;
      case(21) : strcpy(LCD_String_2, "User# 21");       break;
      case(22) : strcpy(LCD_String_2, "User# 22");       break;
      case(23) : strcpy(LCD_String_2, "User# 23");       break;
      case(24) : strcpy(LCD_String_2, "User# 24");       break;
      case(25) : strcpy(LCD_String_2, "User# 25");       break;
      case(26) : strcpy(LCD_String_2, "User# 26");       break;
      case(27) : strcpy(LCD_String_2, "User# 27");       break;
      case(28) : strcpy(LCD_String_2, "User# 28");       break;
      case(29) : strcpy(LCD_String_2, "User# 29");       break;
      case(30) : strcpy(LCD_String_2, "User# 30");       break;
      case(31) : strcpy(LCD_String_2, "User# 31");       break;
      case(32) : strcpy(LCD_String_2, "User# 32");       break;
      case(33) : strcpy(LCD_String_2, "User# 33");       break;
      case(34) : strcpy(LCD_String_2, "User# 34");       break;
      case(35) : strcpy(LCD_String_2, "User# 35");       break;
      case(36) : strcpy(LCD_String_2, "User# 36");       break;
      case(37) : strcpy(LCD_String_2, "User# 37");       break;
      case(38) : strcpy(LCD_String_2, "User# 38");       break;
      case(39) : strcpy(LCD_String_2, "User# 39");       break;
      case(40) : strcpy(LCD_String_2, "User# 40");       break;
   }
}
//******************************************************************************

//******************************************************************************
//    This is the Main function of the program. It sets up the ports as inputs
// and outputs, calls the LCD initialize routine, reads the encoder's User_ID on
// the Rx line (RB1), displays the company name for 2sec then waits for any data
// from the decoder indicating the User_ID. Once the User_ID is accepted, the
// 8-bit binary value will be assigned to a name or number representing the User.

void main(void)
{
   byte User_ID = 0;             // Initialize User_ID value to zero

   OUTPUT_A(0x00);               // Clear all I/O lines
   OUTPUT_B(0x00);

   SET_TRIS_A(0x00);             // Set-up port-A to (b'0000 0000') all outputs
   SET_TRIS_B(0x04);             // Set-up port-B to (b'0000 0100') RB2 = input

   LCD_Init();                   // Initialize LCD (2-lines & 4-bit data entry)
   Clear_LCD();                  // Clear LCD display of any unwanted characters


//-------------- Replace name with your own company or delete it ---------------
   strcpy(LCD_String_1, "LINX"); // Set string values for LCD lines 1 and 2
   strcpy(LCD_String_2, "Technologies Inc");
   LCD_Line_1();                 // Put LCD_String_1 on LCD line 1
   LCD_Line_2();                 // Put LCD_String_2 on LCD line 2
   delay_ms(2000);               // Wait 2S to display company name
//------------------------------------------------------------------------------


Get_User_ID:
   if ((kbhit()) && (User_ID == 0)) // If there is data & User_ID=0 then display
   {
      User_ID = getc();             // Get User_ID
      Clear_LCD();                  // Clear LCD display

      Interpret_User_ID(User_ID);   // Assign 8-bit value to a name or number
      LCD_Line_1();                 // Put LCD_String_1 on LCD line 1
      LCD_Line_2();                 // Put LCD_String_2 on LCD line 2
   }
	else User_ID = 0;                // Clear User_ID
   goto Get_User_ID;                // Keep looping to wait for another User_ID
}
//******************************************************************************


